from abc import ABC, abstractmethod
from pathlib import Path


class Seeder(ABC):
    @abstractmethod
    def truncate(self) -> None:
        pass

    @abstractmethod
    def seed(self, folder: Path) -> None:
        pass


class SeederManager:
    def __init__(self, seeders: list[Seeder]):
        self._seeders = seeders

    def truncate(self) -> None:
        for seeder in self._seeders:
            seeder.truncate()

    def seed(self, folder: Path) -> None:
        for seeder in self._seeders:
            seeder.seed(folder)



from pathlib import Path

import boto3

from seeder.core import SeederManager
from seeder.service.minio_seeder import MinioSeeder

endpoint = "http://127.0.0.1:9000"
username = "admin"
password = "admin-password"

s3_client = boto3.client(
    "s3",
    endpoint_url=endpoint,
    aws_access_key_id=username,
    aws_secret_access_key=password,
)


def test_common_usage(datadir: Path):
    seeder_manager = SeederManager(seeders=[MinioSeeder(endpoint=endpoint, username=username, password=password)])
    seeder_manager.truncate()
    seeder_manager.seed(datadir / "data-1")

    assert _list_all_keys("demo-1") == ["a/b.txt", "c.txt"]
    assert _list_all_keys("demo-2") == ["d.txt"]
    assert _list_all_keys("demo-3") == []


def test_nested_folder(datadir: Path):
    seeder_manager = SeederManager(
        seeders=[
            MinioSeeder(
                endpoint=endpoint,
                username=username,
                password=password,
                get_folder_path=lambda folder, bucket: folder / "minio" / bucket,
            )
        ]
    )
    seeder_manager.truncate()
    seeder_manager.seed(datadir / "data-2")

    assert _list_all_keys("demo-1") == ["a/b/z.txt", "c.txt"]
    assert _list_all_keys("demo-2") == ["d.txt"]
    assert _list_all_keys("demo-3") == []


def _list_all_keys(bucket: str) -> list[str]:
    paginator = s3_client.get_paginator("list_objects_v2")

    outputs: list[str] = []
    for page in paginator.paginate(Bucket=bucket):
        if "Contents" in page:
            for k in [obj["Key"] for obj in page["Contents"]]:
                outputs.append(k)
    return sorted(outputs)



from pathlib import Path

import redis

from seeder.core import Seeder


class RedisSeeder(Seeder):
    def __init__(self, host: str, port: int):
        self._client = redis.Redis(host=host, port=port)

    def truncate(self) -> None:
        self._client.flushall(asynchronous=False)

    def seed(self, folder: Path) -> None:
        pass



import os
from pathlib import Path
from typing import Callable

import boto3

from seeder.core import Seeder


def _default_get_folder_path(folder: Path, bucket: str) -> Path:
    return folder / bucket


class MinioSeeder(Seeder):
    def __init__(
        self,
        endpoint: str,
        username: str,
        password: str,
        get_folder_path: Callable[[Path, str], Path] = _default_get_folder_path,
    ):
        self._client = boto3.client(
            "s3",
            endpoint_url=endpoint,
            aws_access_key_id=username,
            aws_secret_access_key=password,
        )
        self._buckets: list[str] = [info["Name"] for info in self._client.list_buckets()["Buckets"]]
        self._get_folder_path = get_folder_path

    def truncate(self) -> None:
        for bucket in self._buckets:
            self._delete_objects(bucket)

    def seed(self, folder: Path) -> None:
        for bucket in self._buckets:
            local_folder: Path = self._get_folder_path(folder, bucket)
            if not local_folder.exists():
                continue

            for root, dirs, filenames in os.walk(local_folder):
                for filename in filenames:
                    local_filepath = Path(root) / filename
                    key = str(local_filepath.relative_to(local_folder))
                    self._client.upload_file(str(local_filepath), bucket, key)

    def _delete_objects(self, bucket: str) -> None:
        paginator = self._client.get_paginator("list_objects_v2")
        for page in paginator.paginate(Bucket=bucket):
            if "Contents" in page:
                objects = [{"Key": obj["Key"]} for obj in page["Contents"]]
                self._client.delete_objects(Bucket=bucket, Delete={"Objects": objects})
